@@ -28,14 +28,6 @@
 #define TEST(name) static void test_ ## name(void);
 ALL_TESTS
 
-static void
-report_wrong_error(int line)
-{
-	fprintf(stderr, "%d:Wrong error type %d number %d (%#x)\n", line,
-		ct_last_message.type, ct_last_message.number, ct_last_message.number);
-	exit(1);
-}
-
 static CS_CONTEXT *ctx;
 static CS_CONNECTION *conn;
 static CS_COMMAND *cmd;
@@ -71,37 +63,6 @@ TEST_MAIN()
 	return 0;
 }
 
-static void
-_check_fail(const char *name, CS_RETCODE ret, int line)
-{
-	if (ret != CS_FAIL) {
-		fprintf(stderr, "%s():%d: succeeded\n", name, line);
-		exit(1);
-	}
-}
-#define check_fail(func, args) do { \
-	ct_reset_last_message(); \
-	_check_fail(#func, func args, __LINE__); \
-} while(0)
-
-static void
-_check_last_message(ct_message_type type, CS_INT number, const char *msg, int line)
-{
-	bool type_ok = true, number_ok = true, msg_ok = true;
-
-	if (type == CTMSG_NONE && ct_last_message.type == type)
-		return;
-
-	type_ok = (ct_last_message.type == type);
-	number_ok = (ct_last_message.number == number);
-	if (msg && msg[0])
-		msg_ok = (strstr(ct_last_message.text, msg) != NULL);
-	if (!type_ok || !number_ok || !msg_ok)
-		report_wrong_error(line);
-}
-#define check_last_message(type, number, msg) \
-	_check_last_message(type, number, msg, __LINE__)
-
 static void
 test_ct_callback(void)
 {
@@ -546,9 +507,43 @@ static void
 test_cs_convert(void)
 {
 	CS_INT retlen;
-	char outbuf[32];
+	char outbuf[64];
+	typedef struct format_test
+	{
+		CS_INT datatype;
+		CS_INT format;
+		bool success;
+	} format_test;
+
+	static const format_test format_tests[] = {
+		{CS_CHAR_TYPE, CS_FMT_UNUSED, true},
+		{CS_CHAR_TYPE, CS_FMT_NULLTERM, true},
+		{CS_CHAR_TYPE, CS_FMT_PADNULL, true},
+		{CS_CHAR_TYPE, CS_FMT_PADBLANK, true},
+		{CS_CHAR_TYPE, CS_FMT_JUSTIFY_RT, false},
+		{CS_CHAR_TYPE, CS_FMT_STRIPBLANKS, false},
+#if 0
+		/* supported by Sybase, not FreeTDS currently */
+		{CS_CHAR_TYPE, CS_FMT_SAFESTR, true},
+		{CS_CHAR_TYPE, CS_FMT_SUBS_ILL_CHAR, true},
+#endif
+
+		{CS_BINARY_TYPE, CS_FMT_UNUSED, true},
+		{CS_BINARY_TYPE, CS_FMT_NULLTERM, false},
+		{CS_BINARY_TYPE, CS_FMT_PADNULL, true},
+		{CS_BINARY_TYPE, CS_FMT_PADBLANK, false},
+		{CS_BINARY_TYPE, CS_FMT_JUSTIFY_RT, false},
+		{CS_BINARY_TYPE, CS_FMT_STRIPBLANKS, false},
+		{CS_BINARY_TYPE, CS_FMT_SAFESTR, false},
+		{CS_BINARY_TYPE, CS_FMT_SUBS_ILL_CHAR, false},
+
+		{-1, -1, false},
+	};
+	const format_test *p_format_test;
+	CS_NUMERIC num = { 1, 1, {0, 1,} };
 
 	CS_DATAFMT destfmt, srcfmt;
+
 	memset(&srcfmt, 0, sizeof(srcfmt));
 	memset(&destfmt, 0, sizeof(destfmt));
 
@@ -585,6 +580,51 @@ test_cs_convert(void)
 	check_last_message(CTMSG_CSLIB, 0x2010112,
 			   "An illegal value of -1 was placed in the maxlength field of the CS_DATAFMT structure");
 	destfmt.maxlength = 0;
+
+	/* invalid formats */
+	for (p_format_test = format_tests; p_format_test->datatype >= 0; ++p_format_test) {
+		destfmt.datatype = p_format_test->datatype;
+		destfmt.format = p_format_test->format;
+		destfmt.maxlength = 20;
+		srcfmt.maxlength = 4;
+		if (p_format_test->success) {
+			check_call(cs_convert, (ctx, &srcfmt, "3234", &destfmt, outbuf, &retlen));
+		} else {
+			check_fail(cs_convert, (ctx, &srcfmt, "3234", &destfmt, outbuf, &retlen));
+			check_last_message(CTMSG_CSLIB, 0x2010112, "was placed in the format field of the CS_DATAFMT structure");
+		}
+	}
+	destfmt.format = CS_FMT_UNUSED;
+	destfmt.maxlength = 0;
+	srcfmt.maxlength = 0;
+
+	/* test invalid scale/precision for numeric */
+	destfmt.datatype = CS_NUMERIC_TYPE;
+	destfmt.precision = 1;
+	check_call(cs_convert, (ctx, &srcfmt, &num, &destfmt, outbuf, &retlen));
+
+	destfmt.precision = 0;
+	check_fail(cs_convert, (ctx, &srcfmt, &num, &destfmt, outbuf, &retlen));
+	check_last_message(CTMSG_CSLIB, 0x2010112, "was placed in the precision field of the CS_DATAFMT structure");
+
+	destfmt.precision = 78;
+	check_fail(cs_convert, (ctx, &srcfmt, &num, &destfmt, outbuf, &retlen));
+	check_last_message(CTMSG_CSLIB, 0x2010112, "was placed in the precision field of the CS_DATAFMT structure");
+
+	destfmt.precision = 30;
+	destfmt.scale = -1;
+	check_fail(cs_convert, (ctx, &srcfmt, &num, &destfmt, outbuf, &retlen));
+	check_last_message(CTMSG_CSLIB, 0x2010112, "was placed in the scale field of the CS_DATAFMT structure");
+
+	destfmt.scale = 0;
+	check_call(cs_convert, (ctx, &srcfmt, &num, &destfmt, outbuf, &retlen));
+
+	destfmt.scale = 30;
+	check_call(cs_convert, (ctx, &srcfmt, &num, &destfmt, outbuf, &retlen));
+
+	destfmt.scale = 31;
+	check_fail(cs_convert, (ctx, &srcfmt, &num, &destfmt, outbuf, &retlen));
+	check_last_message(CTMSG_CSLIB, 0x2010112, "was placed in the scale field of the CS_DATAFMT structure");
 }
 
 static void
