@@ -41,6 +41,7 @@
 #endif
 
 #include <freetds/tds.h>
+#include <freetds/checks.h>
 #include <freetds/iconv.h>
 #include <freetds/convert.h>
 #include <freetds/bytes.h>
@@ -53,6 +54,8 @@
 #include <syberror.h>
 #include <dblib.h>
 
+#define BCP_DEFAULT_DATEFMT "%Y-%m-%d %H:%M:%S.%z"
+
 #define HOST_COL_CONV_ERROR 1
 #define HOST_COL_NULL_ERROR 2
 
@@ -86,6 +89,7 @@ static int rtrim_u16(uint16_t *str, int len, uint16_t space);
 static STATUS _bcp_read_hostfile(DBPROCESS * dbproc, FILE * hostfile, bool *row_error, bool skip);
 static int _bcp_readfmt_colinfo(DBPROCESS * dbproc, char *buf, BCP_HOSTCOLINFO * ci);
 static int _bcp_get_term_var(const BYTE * pdata, const BYTE * term, int term_len);
+static int _bcp_strftime(char *buf, size_t maxsize, const char *format, const TDSDATEREC * dr, int prec);
 
 /*
  * "If a host file is being used ... the default data formats are as follows:
@@ -621,7 +625,7 @@ bcp_gethostcolcount(DBPROCESS *dbproc)
 
 /** 
  * \ingroup dblib_bcp
- * \brief Set "hints" for uploading a file.  A FreeTDS-only function.  
+ * \brief Set bulk copy options.
  * 
  * \param dbproc contains all information needed by db-lib to manage communications with the server.
  * \param option symbolic constant indicating the option to be set, one of:
@@ -634,12 +638,12 @@ bcp_gethostcolcount(DBPROCESS *dbproc)
  * 		- \b TABLOCK Lock the table
  * 		- \b CHECK_CONSTRAINTS Apply constraints
  * 		- \b FIRE_TRIGGERS Fire any INSERT triggers on the target table
- * \param valuelen The strlen of \a value.  
+ * \param valuelen The length of \a value. Ignored for fixed-length option types.
  * 
  * \return SUCCEED or FAIL.
  * \sa 	bcp_control(), 
  * 	bcp_exec(), 
- * \todo Simplify.  Remove \a valuelen.
+ * \todo Simplify.
  */
 RETCODE
 bcp_options(DBPROCESS * dbproc, int option, BYTE * value, int valuelen)
@@ -776,7 +780,7 @@ _bcp_convert_out(DBPROCESS * dbproc, TDSCOLUMN *curcol, BCP_HOSTCOLINFO *hostcol
 		int prec = curcol->column_prec ? curcol->column_prec : 3;
 
 		tds_datecrack(srctype, src, &when);
-		buflen = (int) tds_strftime((TDS_CHAR *) (*p_data), 256, bcpdatefmt, &when, prec);
+		buflen = _bcp_strftime((char *) *p_data, 256, bcpdatefmt, &when, prec);
 	} else if (srclen == 0 && is_variable_type(curcol->column_type)
 		   && is_ascii_type(hostcol->datatype)) {
 		/*
@@ -841,7 +845,7 @@ bcp_cache_prefix_len(BCP_HOSTCOLINFO *hostcol, const TDSCOLUMN *curcol)
 {
 	int plen;
 
-	if (is_blob_type(hostcol->datatype))
+	if (type_has_textptr(hostcol->datatype))
 		plen = 4;
 	else if (is_numeric_type(hostcol->datatype))
 		plen = 1;
@@ -925,9 +929,13 @@ _bcp_exec_out(DBPROCESS * dbproc, DBINT * rows_copied)
 	tds = dbproc->tds_socket;
 	assert(tds);
 
+	/* See if they want a BCP date format different from the default.
+	 * The ASE bcp client does use the locale for bcp out; but all bcp clients
+	 * appear to successfully read this format for input, regardless of locale.
+	 */
 	bcpdatefmt = getenv("FREEBCP_DATEFMT");
-	if (!bcpdatefmt)
-		bcpdatefmt = "%Y-%m-%d %H:%M:%S.%z";
+	if (bcpdatefmt && !strcmp(bcpdatefmt, BCP_DEFAULT_DATEFMT))
+		bcpdatefmt = NULL;
 
 	if (dbproc->bcpinfo->direction == DB_QUERYOUT ) {
 		if (TDS_FAILED(tds_submit_query(tds, tds_dstr_cstr(&dbproc->bcpinfo->tablename))))
@@ -2429,3 +2437,63 @@ _bcp_free_storage(DBPROCESS * dbproc)
 	dbproc->bcpinfo = NULL;
 }
 
+/** Fast strftime for default bcp format */
+static int
+_bcp_strftime(char *buf, size_t maxsize, const char *format, const TDSDATEREC *dr, int prec)
+{
+	char *pbuf = buf;
+
+	if (format)
+		return (int) tds_strftime(pbuf, maxsize, format, dr, prec);
+
+	pbuf += tds_u32toa_fast(pbuf, dr->year);
+
+	/* The following "& 63" are to avoid overflows if values are corrupted */
+
+	*pbuf++ = '-';
+	tds_02d_fast(pbuf, (dr->month + 1) & 63);
+	pbuf += 2;
+
+	*pbuf++ = '-';
+	tds_02d_fast(pbuf, dr->day & 63);
+	pbuf += 2;
+
+	*pbuf++ = ' ';
+	tds_02d_fast(pbuf, dr->hour & 63);
+	pbuf += 2;
+
+	*pbuf++ = ':';
+	tds_02d_fast(pbuf, dr->minute & 63);
+	pbuf += 2;
+
+	*pbuf++ = ':';
+	tds_02d_fast(pbuf, dr->second & 63);
+	pbuf += 2;
+
+	if (prec > 0) {
+		char ibuf[10];
+
+		*pbuf++ = '.';
+
+		memset(ibuf, '0', sizeof(ibuf));
+		/* Mask off a negative sign (input should not contain this anyway) */
+		tds_u32toa_fast_right(ibuf, dr->decimicrosecond & 0x7FFFFFFF);
+		/* Skip first 3 digits here, as decimicrosecond should not exceed 9,999,999 */
+		memcpy(pbuf, ibuf + 3, prec);
+		pbuf += prec;
+	}
+	*pbuf = '\0';
+
+#if ENABLE_EXTRA_CHECKS
+	{
+		char tbuf[256];
+
+		tds_strftime(tbuf, sizeof tbuf, BCP_DEFAULT_DATEFMT, dr, prec);
+		if (strcmp(tbuf, (char *) buf)) {
+			fprintf(stderr, "_bcp_strftime(%s) does not match tds_strftime(%s)\n", tbuf, (char *) buf);
+			tds_extra_assert(!"_bcp_strftime mismatch");
+		}
+	}
+#endif
+	return (int) (pbuf - buf);
+}
