@@ -45,6 +45,7 @@
 #include <freetds/stream.h>
 #include <freetds/convert.h>
 #include <freetds/utils/string.h>
+#include <freetds/utils/ascii.h>
 #include <freetds/replacements.h>
 #include <freetds/enum_cap.h>
 
@@ -376,6 +377,17 @@ tds7_build_bulk_insert_stmt(TDSSOCKET * tds, TDSPBCB * clause, TDSCOLUMN * bcpco
 	return TDS_SUCCESS;
 }
 
+static const char *
+uc_str(const char *s)
+{
+	size_t n = strcspn(s, "abcdefghijklmnopqrstuvwxyz");
+
+	if (s[n] == '\0')
+		return s;	/* already all uppercase */
+
+	return tds_ascii_strupr(strdup(s));
+}
+
 /**
  * Prepare the query to be sent to server to request BCP information
  * \tds
@@ -392,6 +404,7 @@ tds_bcp_start_insert_stmt(TDSSOCKET * tds, TDSBCPINFO * bcpinfo)
 		TDSCOLUMN *bcpcol;
 		TDSPBCB colclause;
 		char clause_buffer[4096] = { 0 };
+		bool triggers_checked = tds_dstr_isempty(&bcpinfo->hint);	/* Done on demand */
 
 		colclause.pb = clause_buffer;
 		colclause.cb = sizeof(clause_buffer);
@@ -407,8 +420,21 @@ tds_bcp_start_insert_stmt(TDSSOCKET * tds, TDSBCPINFO * bcpinfo)
 				continue;
 			if (!bcpinfo->identity_insert_on && bcpcol->column_identity)
 				continue;
-			if (bcpcol->column_computed)
-				continue;
+			if (bcpcol->column_computed) {
+				if (!triggers_checked) {
+					const char *uc_hint = uc_str(tds_dstr_cstr(&bcpinfo->hint));
+
+					if (strstr(uc_hint, "FIRE_TRIGGERS"))
+						bcpinfo->with_triggers = true;
+
+					if (uc_hint != tds_dstr_cstr(&bcpinfo->hint))
+						free((char *) uc_hint);
+
+					triggers_checked = true;
+				}
+				if (!bcpinfo->with_triggers)
+					continue;
+			}
 			tds7_build_bulk_insert_stmt(tds, &colclause, bcpcol, firstcol);
 			firstcol = 0;
 		}
@@ -467,9 +493,8 @@ tds7_send_record(TDSSOCKET *tds, TDSBCPINFO *bcpinfo,
 		 * identity columns unless indentity_insert is enabled.
 		 */
 
-		if ((!bcpinfo->identity_insert_on && bindcol->column_identity) ||
-			bindcol->column_timestamp ||
-			bindcol->column_computed) {
+		if ((!bcpinfo->identity_insert_on && bindcol->column_identity) || bindcol->column_timestamp
+		    || (bindcol->column_computed && !bcpinfo->with_triggers)) {
 			continue;
 		}
 
@@ -563,7 +588,7 @@ tds5_send_record(TDSSOCKET *tds, TDSBCPINFO *bcpinfo,
 
 	for (i = 0; i < bcpinfo->bindinfo->num_cols; i++) {
 		TDSCOLUMN  *bindcol = bcpinfo->bindinfo->columns[i];
-		if (is_blob_type(bindcol->on_server.column_type)) {
+		if (type_has_textptr(bindcol->on_server.column_type)) {
 			TDSRET rc;
 			/* Elide trailing NULLs */
 			if (bindcol->bcp_column_data->is_null) {
@@ -572,7 +597,7 @@ tds5_send_record(TDSSOCKET *tds, TDSBCPINFO *bcpinfo,
 				for (j = i + 1; j < bcpinfo->bindinfo->num_cols; ++j) {
 					TDSCOLUMN *bindcol2 = bcpinfo->bindinfo->columns[j];
 
-					if (is_blob_type(bindcol2->column_type) && !bindcol2->bcp_column_data->is_null)
+					if (type_has_textptr(bindcol2->column_type) && !bindcol2->bcp_column_data->is_null)
 						break;
 				}
 				if (j == bcpinfo->bindinfo->num_cols)
@@ -804,7 +829,7 @@ tds5_bcp_add_variable_columns(TDSBCPINFO *bcpinfo, tds_bcp_get_col_data get_col_
 
 		/* move the column buffer into the rowbuffer */
 		if (!bcpcol->bcp_column_data->is_null) {
-			if (is_blob_type(bcpcol->on_server.column_type)) {
+			if (type_has_textptr(bcpcol->on_server.column_type)) {
 				cpbytes = 16;
 				bcpcol->column_textpos = row_pos;               /* save for data write */
 			} else if (is_numeric_type(bcpcol->on_server.column_type)) {
@@ -821,7 +846,7 @@ tds5_bcp_add_variable_columns(TDSBCPINFO *bcpinfo, tds_bcp_get_col_data get_col_
 				memcpy(&rowbuffer[row_pos], bcpcol->bcp_column_data->data, cpbytes);
 				tds5_swap_data(bcpcol, &rowbuffer[row_pos]);
 			}
-		} else if (is_blob_type(bcpcol->column_type)) {
+		} else if (type_has_textptr(bcpcol->column_type)) {
 			bcpcol->column_textpos = row_pos;
 		}
 
@@ -925,9 +950,8 @@ tds7_bcp_send_colmetadata(TDSSOCKET *tds, TDSBCPINFO *bcpinfo)
 	num_cols = 0;
 	for (i = 0; i < bcpinfo->bindinfo->num_cols; i++) {
 		bcpcol = bcpinfo->bindinfo->columns[i];
-		if ((!bcpinfo->identity_insert_on && bcpcol->column_identity) || 
-			bcpcol->column_timestamp ||
-			bcpcol->column_computed) {
+		if ((!bcpinfo->identity_insert_on && bcpcol->column_identity) || bcpcol->column_timestamp
+		    || (bcpcol->column_computed && !bcpinfo->with_triggers)) {
 			continue;
 		}
 		num_cols++;
@@ -946,9 +970,8 @@ tds7_bcp_send_colmetadata(TDSSOCKET *tds, TDSBCPINFO *bcpinfo)
 		 * identity columns (unless indentity_insert is enabled
 		 */
 
-		if ((!bcpinfo->identity_insert_on && bcpcol->column_identity) || 
-			bcpcol->column_timestamp ||
-			bcpcol->column_computed) {
+		if ((!bcpinfo->identity_insert_on && bcpcol->column_identity) || bcpcol->column_timestamp
+		    || (bcpcol->column_computed && !bcpinfo->with_triggers)) {
 			continue;
 		}
 
@@ -965,7 +988,7 @@ tds7_bcp_send_colmetadata(TDSSOCKET *tds, TDSBCPINFO *bcpinfo)
 		/* TODO put this in put_info. It seems that parameter format is
 		 * different from BCP format
 		 */
-		if (is_blob_type(bcpcol->on_server.column_type)) {
+		if (type_has_textptr(bcpcol->on_server.column_type)) {
 			converted_name = tds_convert_string(tds, tds->conn->char_convs[client2ucs2],
 							    tds_dstr_cstr(&bcpinfo->tablename),
 							    (int) tds_dstr_len(&bcpinfo->tablename), &converted_len);
@@ -1127,7 +1150,7 @@ tds5_read_bulk_defaults(TDSRESULTINFO *res_info, TDSBCPINFO *bcpinfo)
 		TDS_UCHAR *src = col->column_data;
 		TDS_INT len = col->column_cur_size;
 
-		if (is_blob_type(col->column_type))
+		if (type_has_textptr(col->column_type))
 			src = (unsigned char *) ((TDSBLOB *) src)->textvalue;
 
 		/* find next column having a default */
@@ -1270,7 +1293,7 @@ tds5_get_col_data_or_dflt(tds_bcp_get_col_data get_col_data, TDSBCPINFO *bulk, T
 
 	ret = get_col_data(bulk, bcpcol, offset);
 	coldata = bcpcol->bcp_column_data;
-	if (coldata->is_null && bulk->sybase_colinfo != NULL && !is_blob_type(bcpcol->column_type)) {
+	if (coldata->is_null && bulk->sybase_colinfo != NULL && !type_has_textptr(bcpcol->column_type)) {
 		const TDS5COLINFO *syb_info = &bulk->sybase_colinfo[colnum];
 
 		if (!syb_info->dflt) {
@@ -1344,7 +1367,7 @@ tds_bcp_start_copy_in(TDSSOCKET *tds, TDSBCPINFO *bcpinfo)
 			 * rest can be taken from the server
 			 */
 
-			if (is_blob_type(bcpcol->on_server.column_type))
+			if (type_has_textptr(bcpcol->on_server.column_type))
 				column_bcp_data_size  = 16;
 			else if (is_numeric_type(bcpcol->on_server.column_type))
 				column_bcp_data_size  = tds_numeric_bytes_per_prec[bcpcol->column_prec];
